#!/bin/bash

#############################################################################
################################ config part ################################
#############################################################################

# I had some issues with current ffmpeg versions and the %03d part of the files
# tests with 3.1.7 were OK, 3.3. failed for me
FMPEG=/usr/bin/ffmpeg

# if copy is sufficient
#ENCODE="-c:a copy"
# check available aac encoders with ffmpeg -encoders | grep -i aac
ENCODE="-c:a libfdk_aac -b:a 64k"
#ENCODE="-c:a libfaac -b:a 64k"

WEBBASEDIR="/var/www/html/radio"
TMPDIR="/tmp/radio"
WWWPREFIX="http://[MY PUBLIC IP]/radio/"

JSONFILE=${WEBBASEDIR}/stations.json

#############################################################################
################################ /config part ###############################
#############################################################################

#clean up old stuff
if [ -d ${TMPDIR} ]; then
	rm -rf ${TMPDIR}
fi
mkdir -p ${TMPDIR}

if [ ! -e ${WEBBASEDIR} ]; then
	ln -s ${TMPDIR} ${WEBBASEDIR} 
fi

#copy all stuff to temp dir to avoid disk wear...
SCRIPTDIR=$(dirname $(readlink -f $0))
find ${SCRIPTDIR}/* -prune -type d | while IFS= read -r d; do 
    cp -R ${d} ${TMPDIR}
done

RUNNING=1
# listen to CTRL+C
trap request_stop INT
request_stop() { RUNNING=0; }

# build json with stations
# should be available via /openapi-gateway-app/live/radios
STATIONCOUNT=`find ${TMPDIR} -type f -iname "stream.url" | wc -l`
cat >${JSONFILE} <<EOL
{
  "total_page":1,
  "total_count":${STATIONCOUNT},
  "current_page":0,
  "radios":[
EOL
LASTSTATION=`find ${TMPDIR}/ -type f -iname "stream.url" | tail -n 1`
find ${TMPDIR} -type f -iname "stream.url" -print0 | while IFS= read -r -d $'\0' FILE; do
	OUTPUTDIR=`dirname ${FILE}`
	RADIOID=`basename ${OUTPUTDIR}`
	SEGLIST="${OUTPUTDIR}/pl.m3u8"
	OUTPUT="${OUTPUTDIR}/64%03d.aac"
	STREAMURL=`cat ${FILE}`
	STATIONNAME=`cat ${OUTPUTDIR}/stationname.txt`
	cat >>${JSONFILE} <<EOL
    {
      "id":${RADIOID},
      "kind":"radio",
      "program_name":"${STATIONNAME}",
      "radio_name":"${STATIONNAME}",
      "radio_desc":"",
      "schedule_id":0,
      "support_bitrates":[
        64
      ],
      "rate24_aac_url":"",
      "rate64_aac_url":"${WWWPREFIX}${RADIOID}/pl.m3u8",
      "rate24_ts_url":"",
      "rate64_ts_url":"",
      "radio_play_count":1,
      "cover_url_small":"${WWWPREFIX}${RADIOID}/cover_small.png",
      "cover_url_large":"${WWWPREFIX}${RADIOID}/cover_big.png",
      "updated_at":0,
      "created_at":0
    }
EOL
	if [ ${FILE} != ${LASTSTATION} ]; then
		echo , >> ${JSONFILE}
	fi
done

cat >>${JSONFILE} <<EOL
  ]
}
EOL

echo json file written to ${JSONFILE}
echo make sure it is linked to be accessible like "ln -s ${JSONFILE} /var/www/html/openapi-gateway-app/live/radios"

# clear all aac parts
find ${TMPDIR}/ -type f -iname "*.aac" | xargs rm

# with a dialup connection that gets disconnected every 24h
# we need to re-run ffmpeg in case of disconnected
# - hence the while true loop
while [[ ${RUNNING} > 0 ]]; do
	find ${TMPDIR}/ -type f -iname "stream.url" -print0 | while IFS= read -r -d $'\0' FILE; do
		OUTPUTDIR=`dirname ${FILE}`
		RADIOID=`basename ${OUTPUTDIR}`
		SEGLIST="${OUTPUTDIR}/pl.m3u8"
		OUTPUT="${OUTPUTDIR}/64%03d.aac"
		STREAMURL=`cat ${FILE}`
		STATIONNAME=`cat ${OUTPUTDIR}/stationname.txt`
                PID=`ps axwww | grep ${RADIOID} | grep -v grep | head -n 1 | awk '{print $1}'`
                CURRENTTIME=$(date +%s)
                LASTCHANGED=`find ${OUTPUTDIR} -type f -iname "*.aac" | xargs stat -c "%Y" | sort -r | head -n 1`
                FILEAGE=$((CURRENTTIME-LASTCHANGED))
                if [[ ${FILEAGE} > 7 ]]; then
			echo "  File too old... Killing encoder..."
                       	kill ${PID}
			sleep 1
                fi

		PID=`ps axwww | grep ${RADIOID} | grep -v grep | head -n 1 | awk '{print $1}'`
		echo ${PID} > "${OUTPUTDIR}/ffmpeg.pid"
	
		if [ -z "${PID}" ]; then
			echo Starting ffmpeg for ${STATIONNAME} with ID ${RADIOID}

	                if [[ ${STREAMURL} == *".m3u" ]]; then
	                        STREAMURL=`curl -s ${STREAMURL}`
	                        echo "New stream url from playlist: ${STREAMURL}"
	                fi

	                ${FFMPEG} \
	                        -i "${STREAMURL}" \
	                        ${ENCODE} \
	                        -f ssegment \
	                        -segment_list ${SEGLIST} \
	                        -segment_list_flags +live \
	                        -segment_time 7 \
	                        -segment_list_size 3 \
	                        -segment_wrap 5 \
	                        -segment_list_entry_prefix ${WWWPREFIX}${RADIOID}/ \
	                        "${OUTPUT}" > /dev/null 2>&1 &
			PID=`ps axwww | grep ${RADIOID} | grep -v grep | head -n 1 | awk '{print $1}'`
			echo ${PID} > "${OUTPUTDIR}/ffmpeg.pid"
			echo "  PID ${PID}"
        	fi
	done
	sleep 5
done

echo Stopping all encoders...
find ${TMPDIR} -type f -iname "ffmpeg.pid" -print0 | while IFS= read -r -d $'\0' FILE; do
	PID=`cat ${FILE}`
	if [ ! -z "${PID}" ]; then
		echo "  Stopping ${PID}"
		kill ${PID}
	fi
	rm ${FILE}
done

